﻿using cp.lib;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Linq.Expressions;

namespace TypeMapperUnitTest
{

    /// <summary>
    ///This is a test class for TypeMapperTest and is intended
    ///to contain all TypeMapperTest Unit Tests
    ///</summary>
    [TestClass()]
    public class TypeMapperTest
    {
        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        string guid = Guid.NewGuid().ToString();
        int amount = 9000;
        string type = "Fine";
        DateTime createdType = new DateTime(2000, 10, 10);
        bool isValid = true;

        Model CreatModel()
        {
            return new Model
            {
                Name = guid,
                Amount = amount,
                Type = type,
                CreatedTime = createdType,
                IsValid = isValid
            };
        }

        void AssertViewModel(ViewModel viewModel)
        {
            Assert.AreEqual(viewModel.Name, guid);
            Assert.AreEqual(viewModel.Amount, amount);
            Assert.AreEqual(viewModel.Type, type);
            Assert.AreEqual(viewModel.CreatedTime, createdType);
            Assert.AreEqual(viewModel.IsValid, isValid);
        }

        TypeMapper<Model, ViewModel> CreatMapper()
        {
            var mapper = new TypeMapper<Model, ViewModel>();
            mapper.AutoBuildMap();
            return mapper;
        }

        [TestMethod()]
        public void MapTest1()
        {
            var mapper = CreatMapper();
            var sourceObj = CreatModel();
            var targetObj = new ViewModel();
            mapper.Map(sourceObj, targetObj);
            AssertViewModel(targetObj);
        }

        [TestMethod()]
        public void MapTest2()
        {
            var mapper = CreatMapper();
            var model = CreatModel();
            IEnumerable<Model> sourceObjs = new Model[] { model };
            var targetObjs = new ViewModel[] { new ViewModel() };
            mapper.Map(sourceObjs, targetObjs);
            AssertViewModel(targetObjs[0]);
        }

        [TestMethod()]
        public void MapTest3()
        {
            var mapper = CreatMapper();
            var sourceObj = CreatModel();
            var targetObj = mapper.Map(sourceObj);
            AssertViewModel(targetObj);
        }


        [TestMethod()]
        public void MapTest4()
        {
            var mapper = CreatMapper();
            var model = CreatModel();
            IEnumerable<Model> sourceObjs = new Model[] { model };
            var targetObjs = mapper.Map(sourceObjs);
            var enumerator = targetObjs.GetEnumerator();
            enumerator.MoveNext();
            AssertViewModel(enumerator.Current);
        }


      

        [TestMethod()]
        public void AddMapItemTest()
        {
            var mapper = CreatMapper();
            mapper.AddMapItem(m => m.Amount, v => v.AmountDisp, a => a.ToString("#,###"));
            var model = CreatModel();
            model.Amount = 123456789;
            var vm = mapper.Map(model);
            Assert.AreEqual(vm.AmountDisp, "123,456,789");
        }
    }
}